#!perl
# filename: minimalmarker.pl
# a program for identification of discriminating gene marker sets
# devised by H. FUJII, T. OGATA and M. OMURA; coded in perl by T.OGATA
# National Institute of Fruit Tree Science, Japan
# If you found any bug, let us know at NIFTS_inq@naro.affrc.go.jp
#
# History
# 1 Jul. 2005 first prototype
# 6 Jul. 2005 implemented fast search algorithm mode (-m1)
# 10 Jul. 2005 implemented I/O file and command-line switches
# 11 Jul. 2005 implemented fast check mode for redundancy of markerset (-b1)
# 12 Jul. 2005 implemented function to include or exclude markers (-n -x)
# 28 Jul. 2005 implemented heuristic fast search mode(greedy algorithm) (-m2 -m3 ..)
# 17 Oct. 2005, v1.00
#		implemented search routine using bitwise operation (-b1)
#		implemented double check marker set search mode (-w)
# 10 Aug. 2006, v1.01
#		implemented double check marker set search routine usig bitwise operation
#		implemented double check marker set search routine in heuristic fast search mode
#	1 Aug. 2007, v1.01a
#		changed several messages
#	2 Oct. 2007, v1.02
#		implemented -w3 or over
# 5 Dec, 2014, v1.03
#		minor modification (allow any character for missing data symbol)
#		changed several messages
# 17 Feb, 2015, v1.04
#		implemented maximum number of minimal marker sets (-l)
#		implemented invalid line end error
#		bug fix
# 13 Mar, 2015, v1.04a, v1.04b, v1.04c
#		bug fix introduced in V1.04
#		refactored some codes
#
# Disclaimer
#   MinimalMarker provided by National Agriculture and Food Research Organization
# of Japan (NARO) may be freely distributed, provided that no charge above the 
# cost of distribution is levied, and that the disclaimer below is always attached
# to it.
#  MinimalMarker is provided as is without any guarantees or warranty. Although 
# the author has attempted to find and correct any bugs in the program on MinimalMarker,
# the author is not responsible for any damage or losses of any kind caused by
# the use or misuse of MinimalMarker. The author is under no obligation to provide
# support, service, corrections, or upgrades of MinimalMarker.
#
# Copyright
#  The computer program on MinimalMarker is protected by copyright.
#  Except as specifically permitted, no portion of this software may be distributed
# or reproduced by any means, or in any form, without NARO’s prior written permission.
#
# COPYRIGHT NOTICE
# (c)2006-2011 NARO, Japan. All rights reserved.
#
# Citation
#  When publishing an academic paper using the results obtained using the MinimalMarker,
# please describe the use of MinimalMaker in the paper and cite the following literature.
#  Fujii H., T. Ogata, T. Shimada, T. Endo, H. Iketani, T. Shimizu, T. Yamamoto and M. Omura (2013)
# Minimal marker: an algorithm and computer program for the identification of minimal
# sets of discriminating DNA markers for efficient variety identification.
# Journal of Bioinformatics and Computational Biology 11(2):1250022. DOI: 10.1142/S0219720012500229
#
#Contact
#mailto:NIFTS_inq@naro.affrc.go.jp
#
##############################################################################
# default value of command line option switch
$add_marker = 	0;# (-a)
$blank =			 "";# (-k)
$ignore_char_order = 0;# (-g)
$talkative =		0;# (-t)
$demo = 				0;# (-d)
$print_table =	0;# (-p)
$bitwise_mode = 1;# (-b)
$start_no = 		0;# (-s)
$stop_no =			0;# (-e)
$check_no = 		0;# (-w)
$search_mode =	1;# (-m)
$n_distribution = 0;# (-r)
$maxn_minimal_marker_set = 0;# (-l)

##############################################################################
($enumeration_mode, $fast_mode, $heuristic_mode) = (0..2);
($no_print, $print_only) = (0, 1);
@search_mode_name = ("ENUMERATION MODE", "FAST MODE", "HEURISTIC MODE");
@b_message = ("OFF", "ON");
$disp_limit =  50;
$test = 0;
$n_minimal_marker_set = 0;
$reach_maxn_minimal_marker_set = 0;

# use warnings;
##############################################################################



####################################################
# 							 main routine
####################################################

$| = 1;
open(OUT,">&STDOUT") or die;
&check_param;
&read_data;
&check_n_column;
&bitwise_test;
&print_start_message;
&set_table;
if ($print_table) {
	&print_table;
} else {
	&set_reference;
	if (($search_mode >= $heuristic_mode)) {
		&score_search;
	} else {
		&search_marker_set;
	}
	&print_result;
}
&print_end_message;


###################################################
# 								subroutines
###################################################


sub print_start_message {
	my($bitwise_mode_message, $double_check_mode_message, $search_mode_no, $search_power);
	$search_mode_no = $search_mode;
	if ($search_mode_no >= @search_mode_name) {
		$search_mode_no = @search_mode_name - 1;
	}
	if ($search_mode_no == @search_mode_name - 1) {
		$search_power = " power: $search_mode";
	} else {
		$search_power = "";
	}
	push @start_message, "inputfile:  $input_file\n";
	push @start_message, "outputfile: $output_file\n";
	push @start_message, "number of markers:   $n_marker\n";
	push @start_message, "number of varieties: $n_variety\n";
	push @start_message, "symbol for missing data: $blank\n";
	push @start_message, "marker(s) to be included: ";
	push @start_message, &marker_set2name(@must);
	push @start_message, "marker(s) to be excluded: $mustnot_marker\n";
	push @start_message, "variety(ies) to be discriminated: $limit_variety_name\n";
	push @start_message, "search mode: $search_mode_name[$search_mode_no]$search_power\n";
	push @start_message, "maximum number of marker sets: ";
	if ($maxn_minimal_marker_set > 0) {
		push @start_message, "$maxn_minimal_marker_set";
	}
	push @start_message, "\nminimum number of markers to discriminate each variety: ", $check_no + 1, "\n";
	push @start_message, "bitwise operation mode: $b_message[$bitwise_mode]\n";
	$now_time = localtime;
	$start_time = time;
	push @start_message, "\nstarted at $now_time\n";
	print @start_message;
}


sub print_end_message {
	$end_time = time;
	$now_time = localtime;
	$elapsed_time = $end_time - $start_time;
	push @end_message, "\ncompleted at $now_time\n";
	push @end_message, "elapsed time: $elapsed_time sec.\n\n";
	print @end_message;
	if ($output_file) {
		print OUT @end_message;
	}
}


sub check_param {
	my($i);
	@must = ();# markers that must be included in a marker set
	@mustnot = ();# markers that must be excluded in a marker set
	$output_file = "";
	@limit_variety = ();
	for ($i = 0; $i < @ARGV; ++$i) {
		$_ = $ARGV[$i];
		if (/^-h$/i) {
			print STDERR "program name: minimalmarker\n";
			print STDERR "a program for identification of discriminating gene marker sets\n";
			print STDERR "usage: perl minimalmarker.pl -[switches]\n";
			print STDERR "-h           print this message\n";
			print STDERR "-g           ignore order of characters in each data\n";
			print STDERR "-t           print each marker set instantly after found\n";
			print STDERR "-w[number]   discriminate each variety by at least specified number of markers\n";
			print STDERR "             -w means -w2\n";
			print STDERR "-p           print data table etc.\n";
			print STDERR "-m[0/1/2/n]  search mode; 0:slow(enumeration method), 1:fast(default)\n";
			print STDERR "             2:heuristic(greedy algorithm), n:heuristic with n power(n>2)\n";
			print STDERR "-b[0/1]      bitwise operation; 0: do not use, 1:use(fast; default)\n";
			print STDERR "-d[1/2]      use demo data; 1:Fujii's, 2: in J. Bioinfo. Comput. Biol.\n";
			print STDERR "-k[symbol]   symbol for data not determined (case sensitive)\n";
			print STDERR "-l[number]   maximun number of marker sets \n";
			print STDERR "-v[number]   variety to be discriminated\n";
			print STDERR "-r[number]   number of distribution to calculate on computers\n";
			print STDERR "-a[number]   search marker sets with extra cardinal number\n";
			print STDERR "-s[number]   number of markers start searching\n";
			print STDERR "-e[number]   number of markers stop searching(-1:search all)\n";
			print STDERR "-n[number]   marker to be included in a marker set\n";
			print STDERR "-x[number]   marker to be excluded from a marker set\n";
			print STDERR "-i[filename] input file name\n";
			die 				 "-o[filename] output file name\n";
		} elsif (/^-t$/i) {
			$talkative = 1;
		} elsif (/^-w(\d*)$/i) {
			if ($1 eq "") {
				$check_no = 1;
			} elsif ($1 > 1) {
				$check_no = $1 - 1;
			} else {
				$check_no = 0;
			}
		} elsif (/^-g$/i) {
			$ignore_char_order = 1;
		} elsif (/^-p$/i) {
			$print_table = 2;
		} elsif (/^-m(\d+)$/i) {
			$search_mode = $1;
		} elsif (/^-b(\d)$/i) {
			$bitwise_mode = $1;
		} elsif (/^-d(\d)$/i) {
			$demo = $1;
		} elsif (/^-k(.+)$/i) {
			$blank = $1;
		} elsif (/^-v(\d+)$/i) {
			$limit_variety[$1 - 1] = 1 if ($1 > 0);
		} elsif (/^-r(\d+)$/i) {
			$n_distribution = $1 if ($1 > 1);
		} elsif (/^-a(\d+)$/i) {
			$add_marker = $1;
		} elsif (/^-s(\d+)$/i) {
			$start_no = $1 - 1;
		} elsif (/^-e(\d+)$/i) {
			$stop_no = $1;
		} elsif (/^-l(\d+)$/i) {
			$maxn_minimal_marker_set = $1;
		} elsif (/^-n(\d+)$/i) {
			push(@must, $1 - 1);
			$must_set = join("|", @must);
		} elsif (/^-x(\d+)$/i) {
			push(@mustnot, $1 - 1);
		} elsif (/^-i(.+)$/i) {
				$input_file = $1;
			 # enclose filename in double quotation marks if it includes space.
				open(IN, "< $input_file") or die("Could not open $input_file for reading: $!\n");
		} elsif (/^-o(.+)$/i) {
			$output_file = $1;
			if (open(OUT, "< $output_file")) {
				close(OUT);
				die("output file already exists!\n");
			}
			open(OUT, "> $output_file") or die("Could not open $output_file for writing: $!\n");
		}
	}
	$input_file = "" if ($demo);
}# End of check_param


sub read_data {
	my(@line, @t_must, @limit_variety_list);
	my($i, $j, $v);
	$mustnot_marker = "";
	&demo_data($demo);
	($i, $j) = (0, 0);
	while ($demo or $_ = <IN>) {
		if ($demo) {
			last if ($i > $#t_marker);
			@line = @{$t_marker[$i]};
		} else {
			chomp;
			@line = split(/\,/);
			if ($line[-1] =~ /(\x0d|\x0a)$/) {
				print STDERR "Error! Invalid line end.\n";
				exit;
			}
		}
		if ($line[0] =~ /^Variety$/) {
			@variety = @line;
		} elsif (@line == 0) {
		} elsif ($line[0] =~ /^[^#\;]/) {
			if (grep {$i == $_} @mustnot) {
				push(@mustnot_marker, "$line[0]");
			} else {
				push(@marker_name, shift(@line));
				if ($ignore_char_order) {
					@line = sort_char(@line);
				}
				push(@marker_data, [@line]);
				if (grep {$i == $_} @must) {
					push(@t_must, $j);
				}
				++$j;
			}
			++$i;
		}
	}
	close(IN);
	shift(@variety);
	@must = @t_must;
	$mustnot_marker = join (",", @mustnot_marker);
	$n_marker = @marker_name;# number of markers
	$n_variety = @variety;# number of varieties
	for ($i = 0; $i < $n_variety; ++$i) {
		push(@limit_variety_list, $variety[$i]) if ($limit_variety[$i]);
	}
	$limit_variety_name = join(",", @limit_variety_list);
}# End of read_data


sub sort_char {
	my @t_line = @_;
	my($a, @a, @sa, @r_line);
	foreach $a(@t_line) {
		unless ($a eq $blank) {
			@a = split (/ */, $a);
			@sa = sort @a;
			$a = join ("", @sa);
		}
		push(@r_line, $a);
	}
	return(@r_line);
}# End of sort_char


sub check_n_column {
	my($i);
	if (@marker_name == 0) {
		print STDERR "no data found\n";
		die("perl markerset.pl -h for help\n");
	} elsif (@marker_name > 1) {
		for ($i = 1; $i < @marker_name; ++$i) {
			unless (@{$marker_data[$i - 1]} == @{$marker_data[$i]}) {
				die "Numbers of columns are not same.\n";
			}
		}
	}
}# End of check_n_column


sub bitwise_test {# check bitwise operation limit
	$max_bit = 1;
	until (((1<<$max_bit) == 0) or ((1<<$max_bit) == 1)) {
		++$max_bit;
		if ($max_bit > 1024) {
			$bitwise_mode = 0;
			return;
		}
	}
	if ($max_bit < 8) {
		$bitwise_mode = 0;
		return;
	}
	use integer;
	$n_mbit_set = (($n_marker - 1) / $max_bit) + 1;
#  $top_bit_i = 1<<($max_bit - 1);
	no integer;
	$top_bit = 1<<($max_bit - 1);
}# End of bitwise_test


sub set_table {
	@diff_table = ();
	@diff_sum_list = ();	#識別可能なマーカーの合計数。ゼロの場合は継続検索のためにその組み合わせを削除
	%diff_sum_table = ();	#同上、並び替え用に必須マーカーは削除 V1.04cで配列からハッシュに変更（単純な書き間違え）
	@var_comb =();
	my($i, $j, $k, $m, $a, $b, @cannot, @t_table);
	$k = 0;
	print "preparing tables\n";
	for ($i = 0; $i < ($n_variety - 1); ++$i) {
		for ($j = $i + 1; $j < $n_variety; ++$j) {
#			$diff_sum_list[$k] = 0;
#			$diff_sum_table[$k] = 0;#V1.04cで削除　そもそもハッシュのはず
#			$diff_sum_table{$k} = 0;#V1.04で追加したら動かないのでV1.04bで削除。原因は最後が0の場合ができてしまうため
			if (!@limit_variety or $limit_variety[$i] or $limit_variety[$j]) {
				$diff_sum_list[$k] = 0;#V1.04cでここに移動
				$diff_sum_table{$k} = 0;#V1.04cでここに移動
				for ($m = 0; $m < $n_marker; ++$m) {
					$a = $marker_data[$m][$i];
					$b = $marker_data[$m][$j];
					if (($a eq $b) or ($a eq $blank) or ($b eq $blank)) {#識別不可であれば
						$diff_table[$m][$k] = 0;#テーブル値は0
					} else {#										識別可能であれば
						$diff_table[$m][$k] = 1;#	テーブル値を1にし
						$diff_sum_list[$k]++;#		合計数を+1
						$diff_sum_table{$k}++ unless (grep {$m == $_} @must);# 並べ替え用なので実数は違っても良いか
					}
				}
				$var_comb[$k] = $variety[$i]." x ".$variety[$j];
				if (($diff_sum_list[$k] <= $check_no) and !$print_table) {#	識別可能なマーカー数が基準に達していない場合
					push(@cannot, $var_comb[$k]."\n");#												その品種組み合わせを記録し
					for ($m = 0; $m < $n_marker; ++$m) {#											検索続行用にその列を削除
						pop @{$diff_table[$m]};
					}
					pop @diff_sum_list;
					pop @var_comb;#V1.04cで追加
					delete $diff_sum_table{$k};#V1.04cで追加
					--$k;
				}
				++$k;
			}
		}
	}
	$cn_diff_table = @{$diff_table[0]};# Total column number of @diff_table. @diff_tableだとマーカーの数を返してしまう。
	&sort_table;
	if ($bitwise_mode) {
		&set_b_table;
	}
	print "done\n\n";
	if (@cannot) {
		print STDERR "Error!\n";
		push(@error_message, @cannot);
		push(@error_message, "No marker can discriminates between varieties listed above.\n");
		if (!$print_table) {
			push(@error_message, "Continue searching for marker sets for the other combinations.\n");
		}
		print STDERR @error_message;
#		if ($output_file) {
#			print OUT @start_message;
#			print OUT @result_message;
#		}
#		die("\n");
		print "\n"
	}
}# End of set_table


sub sort_table {# sort tables for fast mode search
	my($i, $m, @t_table);
	@s_diff_index = (sort {$diff_sum_table{$a} <=> $diff_sum_table{$b}} keys %diff_sum_table);
	foreach $m(0..($n_marker - 1)) {
		@t_table = ();
		foreach (@s_diff_index) {
			push(@t_table, $diff_table[$m][$_]);
		}
		push(@s_diff_table, [@t_table]);
	}
	foreach (@s_diff_index) {
		push(@s_diff_sum_list, $diff_sum_list[$_]);
		push(@s_var_comb, $var_comb[$_]);
	}
	if ($n_distribution) {
		@distribution_markers = ();
		for ($i = 0; $i < $cn_diff_table; ++$i) {
			if ($s_diff_sum_list[$i] > $n_distribution) {
				last;
			}
		}
		if ($i > 0) {
			--$i;
			for ($m = 0; $m < $n_marker; ++$m) {
				if ($s_diff_table[$m][$i] == 1) {
					push(@distribution_markers, $m + 1);
				}
			}
			print "list of marker numbers to calculate on up to $n_distribution computers\n";
			print "@distribution_markers\n";
			print "set -n[number] for each calculation\n";
			die("\n");
		} else {
			die "can not distribute to $n_distribution computers\n";
		}
	}
}# End of sort_table


sub set_b_table {
	my($i, $m, $i_bit, $i_set);
	use integer;
	$n_dbit_set = (($cn_diff_table - 1) / $max_bit) + 1;
	no integer;
	for ($m = 0; $m < $n_marker; ++$m) {
		@{$b_diff_table[$m]} = (0) x $n_dbit_set;
		@{$b_s_diff_table[$m]} = (0) x $n_dbit_set;
		for ($i = 0; $i < $cn_diff_table; ++$i) {
			&get_bit_set($i, $i_bit, $i_set);
			$b_diff_all[$i_set] |= (1<<$i_bit);
			if ($diff_table[$m][$i] == 1) {
				$b_diff_table[$m][$i_set] |= (1<<$i_bit);
			}
			if ($s_diff_table[$m][$i] == 1) {
				$b_s_diff_table[$m][$i_set] |= (1<<$i_bit);
			}
		}
	}
}# End of set_b_table


sub print_table {
	my($m, $i);
	&print_item("marker/variety", \@variety, \@marker_data);
	&print_item("marker/varieties", \@var_comb, \@diff_table, "SUM", \@diff_sum_list);
	&print_item("Sorted by SUM", \@s_var_comb, \@s_diff_table, "SUM", \@s_diff_sum_list);
	print OUT "\n";
}# End of print_table


sub print_item {
	my($m, $i);
	print "\n";
	print OUT $_[0];
	foreach $i(@{$_[1]}) {
		print OUT ",".$i;
	}
	print OUT "\n";
	for ($m = 0; $m < $n_marker; ++$m) {
		print OUT $marker_name[$m];
		foreach $i(@{${$_[2]}[$m]}) {
			print OUT ",".$i;
		}
		print OUT "\n";
	}
	if (defined($_[3])) {
		print OUT $_[3];
		foreach $i(@{$_[4]}) {
			print OUT ",".$i;
		}
		print OUT "\n";
	}
	print OUT "\n";
}# End of print_item


sub set_reference {
	undef &is_redundant;
	undef &is_marker_set;
	undef &fast_seek_marker_set;
	if ($bitwise_mode) {
		*is_redundant = \&is_redundant_b;
		if ($check_no == 0) {
			*is_marker_set = \&is_marker_set_b;
			*fast_seek_marker_set = \&fast_seek_marker_set_b;
		} elsif ($check_no == 1) {
			*is_marker_set = \&is_marker_set_b2;
			*fast_seek_marker_set = \&fast_seek_marker_set_b2;
		} else {
			*is_marker_set = \&is_marker_set_c;
			*fast_seek_marker_set = \&fast_seek_marker_set_c;
		}
	} else {
		*is_redundant = \&is_redundant_c;
		*is_marker_set = \&is_marker_set_c;
		*fast_seek_marker_set = \&fast_seek_marker_set_c;
	}
}# End of set_reference


######################################################################################
# main subroutine for enumeration mode(-m0) and fast mode(-m1)
# increase marker set size until discriminating marker set is found
# $li_marker_set: Last index of @marker_set; マーカーセット中のマーカーの数 - 1
# @marker_set: marker set variable; 0番目：最初に選んだmarker, 1番目:次に選んだmarker: ...
# $ci_marker_set: Current index of @marker_set
# enumeration mode calls &seek_marker_set, fast mode calls &fast_seek_marker_set
######################################################################################
#
sub search_marker_set {
	my($i, $j, $m, $limit_no, $i_set, $i_bit);
	print "start searching";
	$n_marker_set = 0;# number of marker set
	$pn_marker_set = 0;# number of marker set at previous loop
	$minimal_marker_set_size = 0;
	@minimal_marker_set = ();# array of minimal marker set
	@b_minimal_marker_set = ();# minimal marker set data for bitwise mode (not filtered)
	@g_minimal_marker_set = ();#	minimal marker set data for conventional mode (not filtered)
	if (&is_marker_set(\@must)) {
		&push_marker_set(\@must);
		$minimal_marker_set_size = @must;
		$n_marker_set = 1;# = @minimal_marker_set;
		return;
	}
	if ($bitwise_mode) {
		@checked_marker_list = (0) x $n_mbit_set;# markers already at former column
		@checked_s_diff_list = (0) x $n_dbit_set;# columns discriminable with current markers
		@checked_s_diff_list_2 = (0) x $n_dbit_set;# used for -w mode bitwise operation
		foreach $m(@must) {
			&get_bit_set($m, $i_bit, $i_set);
			$checked_marker_list[$i_set] |= 1 << $i_bit;
		}
		foreach $m(@must) {
			if ($check_no == 0) {
				for ($i = 0; $i < $n_dbit_set; ++$i) {
					$checked_s_diff_list[$i] |= $b_s_diff_table[$m][$i];# not $checked_b_s_diff_list[$i]
				}
			} else {
				for ($i = 0; $i < $n_dbit_set; ++$i) {
					$checked_s_diff_list[$i] = ($b_s_diff_table[$m][$i] & $checked_s_diff_list_2[$i]) | $checked_s_diff_list[$i];
					$checked_s_diff_list_2[$i] |= $b_s_diff_table[$m][$i];
				}
			}
		}
	} else {
		@checked_marker_list = (0) x $n_marker;
		@checked_s_diff_list = (0) x $cn_diff_table;
		foreach $m(@must) {
			$checked_marker_list[$m] = 1;
			for ($i = 0; $i < $cn_diff_table; ++$i) {
				$checked_s_diff_list[$i] += $s_diff_table[$m][$i];
			}
		}
	}
	$start_no = @must if (@must > $start_no);
	$limit_no = $n_marker;
	for ($li_marker_set = $start_no; $li_marker_set < $limit_no; ++$li_marker_set) {
		print "\nnumber of markers: ".($li_marker_set + 1);
		$is_found = 0;
		@marker_set = @must;# marker set under consideration
		if ($search_mode == $fast_mode) {
			&fast_seek_marker_set(scalar(@must), 1, 0, \@checked_marker_list, \@checked_s_diff_list, \@checked_s_diff_list_2);
		} elsif ($search_mode == $enumeration_mode) {
			&seek_marker_set(scalar(@must), 0);
		}
		$n_marker_set = @minimal_marker_set;
		if ($n_marker_set) {
			if ($minimal_marker_set_size == 0) {
				$minimal_marker_set_size = $li_marker_set + 1;
				$limit_no = $stop_no if ($stop_no >= 0);
				$limit_no = $li_marker_set + $add_marker + 1 if ($add_marker > 0);
				$limit_no = $n_marker if ($limit_no > $n_marker);
			}
		}
		print "; not found" if ($pn_marker_set == $n_marker_set);
		$pn_marker_set = $n_marker_set;
	}
#	&check_marker_set if ($fast_mode);
	print "\nsearching finished\n";
}# End of search_marker_set


##############################################################################
# enumeration mode search routine (try every combination of markers)
# select a marker 
#
# 
# call seek_marker_set recursively with argument 0 
# to avoid duplication, argument 1 is set to current marker number + 1
#
#
#
# 総当たりで検索するアルゴリズムのルーチン
# @marker_setの指定の列（引数０で指定）について引数１で指定したマーカー以降を
# 次々に選択する。最終列だったらマーカーセット条件を満たすかどうかチェックす
# るサブルーチン(is_marker_set)に処理を渡す。最終列でなければ、次の列を選択
# するために、現在の値＋１の引数０でseek_marker_setを再帰で呼び出す。
# この際、マーカーセットの重複を避けるために、引数１には現在のマーカー番号＋１
# を渡している。
##############################################################################
#
sub seek_marker_set {
	my($ci_marker_set, $sp_marker) = @_;
	my($i);
	for ($i = $sp_marker; $i < $n_marker; ++$i) {
		if (grep {$i == $_} @must) {
			next;
		}
		push(@marker_set, $i);
		if ($ci_marker_set == $li_marker_set) {
			if (&is_marker_set(\@marker_set)) {
				&push_marker_set(\@marker_set);
				if ($reach_maxn_minimal_marker_set) {
					return;
				}
			}
		} else {
			&seek_marker_set($ci_marker_set + 1, $i + 1);
			if ($reach_maxn_minimal_marker_set) {
				return;
			}
		}
		pop(@marker_set);
	}
}# End of seek_marker_set


sub is_marker_set_c {# available at any search mode
	my $p_marker_set = $_[0];
	my($i, $j, $s);
	for ($i = 0; $i < $cn_diff_table; ++$i) {
		$s = 0;
		for ($j = 0; $j < @$p_marker_set; ++$j){
			$s += $diff_table[$$p_marker_set[$j]][$i];
			last if ($s > $check_no);
		}
		return(0) if ($j == @$p_marker_set);
	}
	return(1);
}# End of is_marker_set_c


##############################################################################
# bitwise operation version of is_marker_set
# much faster than is_marker_set_c
##############################################################################

sub is_marker_set_b {
	my $p_marker_set = $_[0];
	my($b, $i, $j);
	for ($i = 0; $i < $n_dbit_set; ++$i) {
		$b = 0;
		foreach $m(@$p_marker_set) {
			$b |= $b_diff_table[$m][$i];
		}
		return(0) if ($b != $b_diff_all[$i]);
	}
	return(1);
}# End of is_marker_set_b


sub is_marker_set_b2 {
	my $p_marker_set = $_[0];
	my($b, $b2, $i, $j);
	for ($i = 0; $i < $n_dbit_set; ++$i) {
		$b = 0;
		$b2 = 0;
		foreach $m(@$p_marker_set) {
			$b = ($b_diff_table[$m][$i] & $b2) | $b;
			$b2 |= $b_diff_table[$m][$i];
		}
		return(0) if ($b != $b_diff_all[$i]);
	}
	return(1);
}# End of is_marker_set_b2


######################################################################################
# fast mode search routine
# 高速検索アルゴリズムのルーチン
# 合計値が小さい列から順にその列を識別できるマーカーのみ選択していく。
#
# 具体的な処理の概要は以下の通り（自分で忘れないために書いており、非常にわかりにくい説明）
# 1. @diff_table を最終行の合計値で昇順にソート（@s_diff_table）
# 2. @s_diff_tableおよび@checked_s_diff_listをもとに、現在選択しているマーカーで識別で
# 	 きない列のうち、合計値が最小の列を一つ選ぶ（列Ｍとする）
# 3. 列Ｍの値が１のマーカーを一つ選択し、そのマーカーで識別できる列について
# 	 @checked_s_diff_listをコピーした@t_checked_s_diff_listと加算する（列Ｍに複数のマー
# 	 カーがある場合に対応するために、@checked_s_diff_listの値は変更しない。）。
# 4. 現在の選択マーカー数が検索中のマーカー数に達していれば最小マーカーセットの条件を
# 	 満たすか確認。達していなければ、再帰呼び出しでマーカー数＋１の処理に移る。
# 	 この場合、引数には@marker_setのカラム位置（すなわち選択マーカー数-1）、@s_diff_table
# 	 の現インデックス（そのインデックス以下の@s_diff_tableの列は既選択マーカーで全て識
# 	 別可）、@t_checked_s_diff_listを渡す。
#
# 検索対象マーカーセットにまだ重複があるので、その除外処理を追加すればもう少し速くなる
# 可能性がある。上位の桁で（候補ではなく）既に試したマーカーを下位の桁の候補からはずす
# （７月１１日実装済み）。とりあえず、私の頭で実現できる重複回避は実装完了。まだ重複は
# あるが回避方法がわからない。最小マーカーセット以外を検索しない限り重複はほとんど発生
# しないし、条件で異なるだろうが現時点でも重複は全体の２割以下なのでこれ以上この問題に
# はつっこまない。
#
######################################################################################
#
sub fast_seek_marker_set_c {
	my($ci_marker_set, $tmp, $i_s_diff_table, $p_checked_marker_list, $p_checked_s_diff_list) = @_;
	my($i, $m, @marker_list, @t_checked_s_diff_list, @t_checked_marker_list, @t_marker_set);
	@t_checked_marker_list = @$p_checked_marker_list;
	while ($$p_checked_s_diff_list[$i_s_diff_table] > $check_no) {#select column of smallest sum not yet checked
		++$i_s_diff_table;
	}
#  if ($i_s_diff_table >= $cn_diff_table) {
# 	 return(0);
#  }# not necessary
	for ($m = 0; $m < $n_marker; ++$m) {# pick out markers that can discriminate selected column
		if (($s_diff_table[$m][$i_s_diff_table] == 1) and ($t_checked_marker_list[$m] == 0)) {
			push(@marker_list, $m);
		}
	}
	foreach $m(@marker_list) {
		push(@marker_set, $m);
		if ($ci_marker_set == $li_marker_set) {
			if (&fast_is_marker_set_c($m, $i_s_diff_table, \@$p_checked_s_diff_list)) {
				&push_marker_set(\@marker_set);
				if ($reach_maxn_minimal_marker_set) {
					return;
				}
			}
		} else {
			@t_checked_s_diff_list = @$p_checked_s_diff_list;
			$t_checked_marker_list[$m] = 1;
			for ($i = $i_s_diff_table; $i < $cn_diff_table; ++$i) {
					$t_checked_s_diff_list[$i] += $s_diff_table[$m][$i];
			}
			&fast_seek_marker_set_c($ci_marker_set + 1, 0, $i_s_diff_table, \@t_checked_marker_list, \@t_checked_s_diff_list);
			if ($reach_maxn_minimal_marker_set) {
				return;
			}
		}
		pop(@marker_set);
	}
}# End of fast_seek_marker_set_c


sub fast_is_marker_set_c {
	my($last_marker, $i_s_diff_table, $p_checked_s_diff_list) = @_;
	my($i);
	for ($i = $i_s_diff_table; $i < $cn_diff_table; ++$i) {
		if ($$p_checked_s_diff_list[$i] + $s_diff_table[$last_marker][$i] <= $check_no) {
			return(0);# どれか一つでも識別できない列が見つかったら条件を満たさない
		}# As @t_checked_s_diff_listがあるので@marker_setについては最後のマーカーだけ調べればよい。
	}
	return(1);
}# End of fast_is_marker_set_c


##############################################################################
# bitwise mode version of fast search algorithm routine
##############################################################################


sub fast_seek_marker_set_b {
	my($ci_marker_set, $i_bit, $i_set, $p_checked_b_marker_list, $p_checked_b_s_diff_list) = @_;
	my($i, $m, $t_bit, $t_set, @marker_list, @t_marker_set, @t_checked_b_s_diff_list);
	my @t_checked_b_marker_list = @$p_checked_b_marker_list;
LBL: for (; $i_set < $n_dbit_set; ++$i_set) {
		while (1) {
			last LBL if !($$p_checked_b_s_diff_list[$i_set] & $i_bit);
			last if ($i_bit == $top_bit);
			$i_bit <<= 1;
		}
		$i_bit = 1;
	}
	for ($m = 0; $m < $n_marker; ++$m) {# 未チェック列を識別可能なマーカーを候補として処理
		if (($b_s_diff_table[$m][$i_set] & $i_bit) and !(&is_b_in_list($m, @t_checked_b_marker_list))) {
			push(@marker_list, $m);
		}
	}
	foreach $m(@marker_list) {
		push(@marker_set, $m);
		if ($ci_marker_set == $li_marker_set) {
			if (&fast_is_marker_set_b($m, $i_set, \@$p_checked_b_s_diff_list)) {
				&push_marker_set(\@marker_set);
				if ($reach_maxn_minimal_marker_set) {
					return;
				}
			}
		} else {
			@t_checked_b_s_diff_list = @$p_checked_b_s_diff_list;
			&get_bit_set($m, $t_bit, $t_set);
			$t_checked_b_marker_list[$t_set] |= 1<<$t_bit;
			for ($i = $i_set; $i < $n_dbit_set; ++$i) {
				$t_checked_b_s_diff_list[$i] |= $b_s_diff_table[$m][$i];
			}
			&fast_seek_marker_set_b($ci_marker_set + 1, $i_bit, $i_set, \@t_checked_b_marker_list, \@t_checked_b_s_diff_list);
			if ($reach_maxn_minimal_marker_set) {
				return;
			}
		}
		pop(@marker_set);
	}
}# End of fast_seek_marker_set_b


sub fast_is_marker_set_b {
	my($last_marker, $i_set, $p_checked_b_s_diff_list) = @_;
	my($i);
	for ($i = $i_set; $i < $n_dbit_set; ++$i) {
		if (($b_s_diff_table[$last_marker][$i] | $$p_checked_b_s_diff_list[$i]) != $b_diff_all[$i]) {
			return(0);# どれか一つでも識別できない列が見つかったら条件を満たさない
		}# @t_checked_b_s_diff_listがあるので@marker_setについては最後のマーカーだけ調べればよい。
	}
	return(1);
}# End of fast_is_marker_set_b


##############################################################################
# bitwise and -w mode version of fast search algorithm routine
##############################################################################

sub fast_seek_marker_set_b2 {
	my($ci_marker_set, $i_bit, $i_set, $p_checked_b_marker_list, $p_checked_b_s_diff_list, $p_checked_b2_s_diff_list) = @_;
	my($i, $m, $t_bit, $t_set, @marker_list, @t_marker_set, @t_checked_b_s_diff_list, @t_checked_b2_s_diff_list);
	my @t_checked_b_marker_list = @$p_checked_b_marker_list;
LBL: for (; $i_set < $n_dbit_set; ++$i_set) {
		while (1) {
			last LBL if !($$p_checked_b_s_diff_list[$i_set] & $i_bit);
			last if ($i_bit == $top_bit);
			$i_bit <<= 1;
		}
		$i_bit = 1;
	}
	for ($m = 0; $m < $n_marker; ++$m) {# 未チェック列を識別可能なマーカーを候補として処理
		if (($b_s_diff_table[$m][$i_set] & $i_bit) and !(&is_b_in_list($m, @t_checked_b_marker_list))) {
			push(@marker_list, $m);
		}
	}
	foreach $m(@marker_list) {
		push(@marker_set, $m);
		if ($ci_marker_set == $li_marker_set) {
			if (&fast_is_marker_set_b2($m, $i_set, \@$p_checked_b_s_diff_list,\@$p_checked_b2_s_diff_list)) {
				&push_marker_set(\@marker_set);
				if ($reach_maxn_minimal_marker_set) {
					return;
				}
			}
		} else {
			@t_checked_b_s_diff_list = @$p_checked_b_s_diff_list;
			@t_checked_b2_s_diff_list = @$p_checked_b2_s_diff_list;
			&get_bit_set($m, $t_bit, $t_set);
			$t_checked_b_marker_list[$t_set] |= 1<<$t_bit;
			for ($i = $i_set; $i < $n_dbit_set; ++$i) {
				$t_checked_b_s_diff_list[$i] = ($b_s_diff_table[$m][$i] & $t_checked_b2_s_diff_list[$i]) | $t_checked_b_s_diff_list[$i];
				$t_checked_b2_s_diff_list[$i] |= $b_s_diff_table[$m][$i];
			}
			&fast_seek_marker_set_b2($ci_marker_set + 1, $i_bit, $i_set, \@t_checked_b_marker_list, \@t_checked_b_s_diff_list, \@t_checked_b2_s_diff_list);
			if ($reach_maxn_minimal_marker_set) {
				return;
			}
		}
		pop(@marker_set);
	}
}# End of fast_seek_marker_set_b2


sub fast_is_marker_set_b2 {
	my($last_marker, $i_set, $p_checked_b_s_diff_list, $p_checked_b2_s_diff_list) = @_;
	my($i);
	for ($i = $i_set; $i < $n_dbit_set; ++$i) {
		if ((($b_s_diff_table[$last_marker][$i] & $$p_checked_b2_s_diff_list[$i]) | $$p_checked_b_s_diff_list[$i]) != $b_diff_all[$i]) {
			return(0);# どれか一つでも識別できない列が見つかったら条件を満たさない
		}# @t_checked_b_s_diff_listがあるので@marker_setについては最後のマーカーだけ調べればよい。
	}
	return(1);
}# End of fast_is_marker_set_b2


sub is_b_in_list {
	my($i_bit, $i_set);
	&get_bit_set($_[0], $i_bit, $i_set);
	$_[++$i_set] & 1<<$i_bit;
}# End of is_b_in_list



####################################################################################
# 簡易高速検索法 -m2 -m3
# 検索速度は速いが、必ず最小のマーカーセットが検出されるとは限らない。最小マーカー
# セットが検出される場合も検出されるのはサブセットで、全ての最小マーカーセットが検出
# されるわけではない。
# 上記の制限はあるものの、意外と多くの最小マーカーセットを検出できる。このため
#マーカー数、品種数が多い場合にとりあえず候補を得る目的では有効と思われる。
#
# アルゴリズムの説明
# 基本的には欲張りアルゴリズム(greedy algorithm)である。
# まずどれか一つマーカーを選択する。次に、そのマーカーで識別できないdiff_tableの列を
# 最も多く識別できるマーカーを上位から数個選択する（-m2では2個、-m3では3個）。次に、
# それらのマーカーで識別できないdiff_table の列を最も多く識別できるマーカーを同様に
# 選択する。全ての列が識別可能になるまでこれを繰り返す。
# 最初に選ぶマーカーを順次変更していき、全てのマーカーについて処理を行う。
# これまでよりも少ない数のマーカーセットが見つかった場合は、既に見つかっているマーカー
# セットを破棄する。既に見つかっているマーカーセットのマーカー数を超えたら現在の検索
# 処理を中止し、次の検索処理に移行する。
# 現在は最小マーカーセットしか検出しないが、マーカー数の上限を決めて、それ以下であれ
# ば全てリストアップするように変更することは可能
####################################################################################
#

sub score_search {
	my($m, $n, $i);
	my @sort_score = ();
	$is_found = 1;
	$talkative = 0;
	$show_progress = 1;
	@all_marker = (0..($n_marker - 1));
	$minimal_marker_set_size = $n_marker;
	&print_scale;
	$p_progress = 0;
	my @rest_table = (0..$cn_diff_table - 1);
	$new_marker = "-";
	&count_marker_score(\@all_marker, \@sort_score, \@rest_table);
	for ($n = 0; $n < $n_marker; ++$n) {
		$m = $sort_score[$n];
		&print_progress($n, $n_marker);
		@marker_set = ($m);
		@rest_table = ();# @marker_setにあるマーカーで識別できないdiff_tableの列番号
		@diff_count_list = (0) x $cn_diff_table;
		for ($i = 0; $i < $cn_diff_table; ++$i) {
			$diff_count_list[$i] += $diff_table[$m][$i];
			if ($diff_count_list[$i] <= $check_no) {
				push(@rest_table, $i);
			}
		}
		&next_score(\@marker_set, \@rest_table, \@diff_count_list);
	}
	&print_progress($n_marker, $n_marker);
	print "\n";
	$n_marker_set = @minimal_marker_set;
}# End of score_search


sub next_score {
	my($p_marker_set, $p_rest_table, $p_diff_count_list) = @_;
	my($i, $j, @sort_score, @rest_table, @rest_marker, @t_diff_count_table);
	my @t_marker_set = @$p_marker_set;
	my @t_rest_table = @$p_rest_table;
	if (&is_marker_set(\@t_marker_set)) {
		if ((@must == 0) or (grep(/^($must_set)$/, @t_marker_set)  == @must)) {
			$new_marker = "+" if ($new_marker ne "*");
			if (@t_marker_set < $minimal_marker_set_size) {
				$minimal_marker_set_size = @t_marker_set;
				@minimal_marker_set = ();
				$new_marker = "*";
			}
			&push_marker_set(\@t_marker_set);
		}
#		return(1);
	} else {
		return(0) if (@t_marker_set >= $minimal_marker_set_size);
		@sort_score = ();
		@rest_marker = grep(!/^@t_marker_set$/, @all_marker);
		&count_marker_score(\@rest_marker, \@sort_score, \@t_rest_table);
		my $n_nodes = $search_mode;
		$n_nodes = @sort_score if ($n_nodes > @sort_score);
		for ($i = 0; $i < $n_nodes; $i++) {
			@marker_set = (@t_marker_set, $sort_score[$i]);
			@t_diff_count_list = @$p_diff_count_list;
			@rest_table = ();
			foreach $j(@t_rest_table) {
				$t_diff_count_list[$j] += $diff_table[$sort_score[$i]][$j];
				if ($t_diff_count_list[$j] <= $check_no) {
					push(@rest_table, $j);
				}
			}
			&next_score(\@marker_set, \@rest_table, \@t_diff_count_list);
#			if (&next_score(\@marker_set, \@rest_table, \@t_diff_count_list) == 1) {
#				if ($i == ($n_nodes + 1)) {
#					$n_nodes++;
#				}
#			}
		}
	}
}# End of next_score


sub count_marker_score {
	my($m, $i, %marker_score);
	my($t_rest_marker, $p_sort_score, $t_rest_table) = @_;
	%marker_score = ();
	foreach $i(@$t_rest_table) {
		foreach $m(@$t_rest_marker) {
			if ($diff_table[$m][$i] == 1) {
				++$marker_score{$m};
			}
		}
	}
	@$p_sort_score = (sort {$marker_score{$b} <=> $marker_score{$a}} keys %marker_score);
}# End of count_marker_score


sub print_scale {
	if ($show_progress) {
		print "\n0  10   20   30   40   50   60   70   80   90  100%\n";
		print "|---+----+----+----+----+----+----+----+----+----|\n";
	}
}


sub print_progress {
	my($part, $total) = @_;
	my($i);
	if ($show_progress) {
		use integer;
		$progress = ($part * 50) / $total;
		no integer;
		for ($i = $p_progress; $i < $progress; ++$i) {
			print "$new_marker";
		}
		$new_marker = "-";
		$p_progress = $progress;
	}
}# End of print_progress;


###################################################################################
# 以下は各検索モードに特異でないサブルーチン
###################################################################################

########################################################################
# 検出されたマーカーセットについて、冗長でないか等条件を満たすか確認し、
# 満たす場合はマーカーセットリストに追加する。
########################################################################
#
sub push_marker_set {
	my($p_marker_set) = @_;
	my($i, @t_marker_set, @t_must);
	if (&is_redundant(@$p_marker_set)) {
		return(0);
	}
# -m0/-m1時に-nで必須のマーカーを設定した場合、または-sで開始マーカーセット数を指定した場合、
# 最小マーカーセットのマーカー数よりもマーカー数が大きいと、最小マーカーセットに別のマーカー
# が加わっただけの冗長なマーカーセットも検出される。以下はそれを排除する処理

	if ((@must and ($search_mode <= $fast_mode)) or ($start_no != 0)) {
		if ($start_no != 0) {
			@t_must = @$p_marker_set;
		} else {
			@t_must = @must;
		}
		foreach $mm(@t_must) {
			@t_marker_set = grep {$mm != $_} @$p_marker_set;
			if (&is_marker_set(\@t_marker_set)) {
				return(0);
			}
		}
	}
	@t_marker_set = sort{$a <=> $b}(@$p_marker_set);# マーカーセット内を昇順にする。
	push(@minimal_marker_set, [@t_marker_set]);
	if ($talkative) {
		print "\nmarker set found: ";
		print &marker_set2name(@t_marker_set);
		print "continue searching";
	} elsif ($is_found == 0) {
		print "; found";
		$is_found = 1;
	}
	$n_minimal_marker_set = @minimal_marker_set;
	if (($maxn_minimal_marker_set > 0) and ($n_minimal_marker_set >= $maxn_minimal_marker_set)) {
		$reach_maxn_minimal_marker_set = 1;
	}
	return($n_minimal_marker_set); #最小マーカーセットの数を返す。今は使っていない。
}# End of push_marker_set


######################################################################################
# 冗長なマーカーセット（既存マーカーセットに別のマーカーが加わっただけ、もしくは全く
# 同じマーカーの組み合わせ；高速モードでは後者も検出される）の排除
# 正攻法；grepを使って比較（ハッシュを使用するよりは速い）。正攻法だが芸がない
######################################################################################
#
sub is_redundant_c {
	my @t_marker_set = @_; 
	my($s) = join( "|", @t_marker_set);
	foreach $mms(@g_minimal_marker_set) {
		if (grep(/^($s)$/, @{$mms}) == @{$mms}) {
			return(1);
		}
	}
	push(@g_minimal_marker_set, [@t_marker_set]);
	return(0);
}# End of is_redundant_c


######################################################################################
# 冗長なマーカーセットの排除ルーチンその２
# ビット演算を使用する方法。grepやハッシュを使う方法と比べると極めて速い。ただし検出
# されるマーカーセットが少ない場合はこのルーチンが呼ばれる回数が少ないので全体の実行
# 速度にはほとんど違いはない。-aスイッチを使う場合等検出されるマーカー数が多い場合は
# 劇的に速くなる。
######################################################################################
#
sub is_redundant_b {
	my($i, $j, $i_bit, $i_set, @b_marker_set);
	@b_marker_set = (0) x $n_mbit_set;
	foreach $ms(@_) {
		&get_bit_set($ms, $i_bit, $i_set);
		$b_marker_set[$i_set] |= (1<<$i_bit);
	}
	foreach $bmms(@b_minimal_marker_set) {
		for ($j = 0; $j < $n_mbit_set; ++$j) {
			unless (($b_marker_set[$j] & ${$bmms}[$j]) == ${$bmms}[$j]) {
				last;
			}
		}
		if ($j == $n_mbit_set) {
			return(1);
		}
	}
	push(@b_minimal_marker_set, [@b_marker_set]);
	return(0);
}# End of is_redundant_b


sub get_bit_set {
	use integer;
	$_[1] = $_[0] % $max_bit;
	$_[2] = $_[0] / $max_bit;
	no integer;
}# End of get_bit_set


##################################################################################
# 検索結果の表示
##################################################################################
#
sub print_result {
	my($i, $mms, @marker_set_sorted_list);
	unless ($n_marker_set) {
		print "No marker set was found.\n";
	} else {
		if (($n_marker_set > $disp_limit) and !($output_file)) {
			if (open(OUT, ">> markerset.tmp")) {
				print "There are more than $disp_limit markersets found.\n";
				print "The result is written or added to markerset.tmp.\n";
				$output_file = "markerset.tmp";
			}
		}
		push(@result_message, "\nnumber of marker sets found: $n_marker_set\n");
		print @result_message;
		if ($output_file) {
			print OUT @start_message;
			print OUT @error_message;
			print OUT @result_message;
		}
		&sort_marker_set_list(\@marker_set_sorted_list);
		print OUT "list of marker sets found:\n";
		foreach $i(@marker_set_sorted_list) {
			print OUT &marker_set2name(@{$minimal_marker_set[$i]});
		}
	}
}# End of print_result


sub sort_marker_set_list {
	my $sort_score = $_[0];
	my($i, $j, $mms, %score);
	$i = 0;# 高速検索ではマーカーセットが順不同に検出されるので、リストを昇順にソートする。
	foreach  $mms(@minimal_marker_set) {
# 	 @{$mms} = sort{$a <=> $b}(@{$mms});# マーカーセット内を昇順にする。
		$score{$i} = 0;
		for ($j = 1; $j <= @{$mms}; ++$j) {
			$score{$i} += ((${$mms}[-$j] + 1) * ($n_marker ** $j));
		}
		++$i;
	}
	@$sort_score = (sort {$score{$a} <=> $score{$b}} keys %score);
}# End of sort_marker_set_list


sub check_marker_set {
	foreach (@minimal_marker_set) {
		if (!&is_marker_set_c(\@$_)) {
			print STDERR "Sorry. A bug was found in fast mode. Use enumeration mode or heuristic mode.\n";
			die "Let us know the detail.\n";
		}
	}
}# End of check_marker_set


###############################################################################
# 引数で指定したマーカーセット配列変数（@marker_set等）の内容をマーカー名にして
# 返す。
# マーカーセット配列変数はマーカー名変数@marker_nameへインデックスxを保持している。
# このマーカー名をコンマでつないで返す。
###############################################################################
#
sub marker_set2name {
	my @t_name = ();
	foreach (@_) {
		push(@t_name, $marker_name[$_]);
	}
	join(",", @t_name)."\n";
}# End of marker_set2name


sub demo_data {
	my $demo = $_[0];
	if ($demo == 1) {# 藤井さんのテストデータ
 # 最初の列はマーカー名
		@variety = (undef,V1,V2,V3,V4,V5,V6,V7,V8,V9);
		@t_marker = (
			[M1,1,0,1,0,1,1,1,1,1],
			[M2,AA,AB,BB,AA,AB,AB,AA,AB,BB],
			[M3,0,1,1,1,0,0,1,0,1],
			[M4,0,1,0,0,1,0,0,1,1],
			[M5,AB,AB,BB,AA,AA,AA,AB,AA,AA],
			[M6,0,1,1,0,0,1,0,0,1],
			[M7,1,1,1,1,0,1,1,1,1],
			[M8,1,0,0,0,1,0,1,1,0]
# 		,[M9,A,B,C,D,E,F,G,H,I]
		);
	} elsif ($demo == 2) {# J. Bioinform. Comput. Biol. 3(2):267-79 のテストデータ
# 本文は23varietiesとなっているが表には品種4がなくて22品種。どちらかが間違っている。
		@variety = (undef,V1,V2,V3,V4,V5,V6,V7,V8,V9,V10,V11,V12,V13,V14,V15,V16,V17,V18,V19,V20,V21,V22);
		@t_marker = (
			[A,1,0,1,1,0,0,0,1,0,1,1,1,0,1,1,1,1,0,0,0,1,1],
			[B,1,0,1,1,0,0,0,1,0,1,0,0,1,0,1,1,0,0,0,0,1,1],
			[C,0,0,1,0,1,0,1,1,0,0,0,1,0,1,0,1,1,0,1,0,0,0],
			[D,1,1,1,1,1,0,1,1,0,1,1,1,1,0,1,1,0,1,0,0,1,1],
			[E,1,0,1,1,0,0,0,0,0,0,1,0,0,0,1,1,0,0,0,0,1,1],
			[F,0,1,0,0,1,1,0,0,1,0,0,0,1,0,0,0,0,0,0,1,0,0],
			[G,1,0,1,1,0,0,0,1,0,1,1,1,0,0,1,0,0,1,1,0,1,0],
			[H,0,0,0,0,1,1,1,0,1,0,0,0,0,0,0,0,0,0,1,1,0,0],
			[I,0,0,1,1,1,0,1,1,0,1,0,0,0,1,1,1,0,1,1,0,1,1],
			[J,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,1,0,0,0,0,0],
			[K,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,1,0,1,1,0,1,1],
			[L,0,1,0,0,0,1,0,0,0,0,0,0,1,0,0,0,1,0,0,1,0,0],
			[M,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,0],
			[N,1,1,1,1,0,1,1,1,1,1,1,0,1,0,0,1,0,0,1,1,1,0],
			[O,0,0,0,0,1,1,1,0,0,0,0,1,0,1,0,0,1,0,1,1,0,0],
			[P,1,1,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0,0,1,0,0],
			[Q,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,0,1],
			[R,1,0,1,1,0,1,0,1,0,1,1,0,0,0,1,1,1,1,1,0,1,1],
			[S,0,1,1,0,1,0,1,1,0,0,1,1,1,1,1,1,1,1,0,0,0,0],
			[T,0,0,0,1,0,0,0,0,1,0,0,0,1,0,0,1,0,0,1,1,1,1],
			[U,0,0,0,0,1,0,0,1,0,0,0,1,0,1,0,1,1,0,1,1,1,0],
			[V,0,1,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,1,0,1,0,0]
		);
	}
}


##############################################
# for debugging
##############################################

sub print_bit {
	my @bit_array = @_;
	my $bit;
	foreach $bit_char(@bit_array) {
		for ($i = 0; $i < $max_bit; ++$i) {
# 	 for ($bit = 1; $bit != 0; $bit<<1) {
			$bit = 1<<$i;
			if ($bit & $bit_char) {
				print "1";
			} else {
				print "0";
			}
		}
	}
	print "\n";
}
